@@ -17,18 +17,23 @@
 """Tree classes, representing directory at point in time.
 """
 
-from __future__ import absolute_import
+from collections import deque
 
 import os
 import re
+from typing import Type, TYPE_CHECKING, Optional
+
 
 from .. import (
     branch as _mod_branch,
+    controldir,
     debug,
     errors,
     lazy_import,
     osutils,
     revision,
+    trace,
+    transport as _mod_transport,
     )
 from ..controldir import (
     ControlDir,
@@ -45,26 +50,75 @@ from ..revisiontree import (
 lazy_import.lazy_import(globals(), """
 from breezy import (
     add,
-    controldir,
-    trace,
-    transport as _mod_transport,
     )
 from breezy.bzr import (
     inventory as _mod_inventory,
     )
 """)
-from ..sixish import (
-    viewvalues,
-    )
 from ..tree import (
     FileTimestampUnavailable,
     InterTree,
     MissingNestedTree,
     Tree,
     TreeChange,
+    TreeFile,
     )
 
 
+class InventoryTreeChange(TreeChange):
+
+    __slots__ = TreeChange.__slots__ + ['file_id', 'parent_id']
+
+    def __init__(self, file_id, path, changed_content, versioned, parent_id,
+                 name, kind, executable, copied=False):
+        self.file_id = file_id
+        self.parent_id = parent_id
+        super(InventoryTreeChange, self).__init__(
+            path=path, changed_content=changed_content, versioned=versioned,
+            name=name, kind=kind, executable=executable, copied=copied)
+
+    def __repr__(self):
+        return "%s%r" % (self.__class__.__name__, self._as_tuple())
+
+    def _as_tuple(self):
+        return (self.file_id, self.path, self.changed_content, self.versioned,
+                self.parent_id, self.name, self.kind, self.executable, self.copied)
+
+    def __eq__(self, other):
+        if isinstance(other, TreeChange):
+            return self._as_tuple() == other._as_tuple()
+        if isinstance(other, tuple):
+            return self._as_tuple() == other
+        return False
+
+    def __lt__(self, other):
+        return self._as_tuple() < other._as_tuple()
+
+    def meta_modified(self):
+        if self.versioned == (True, True):
+            return (self.executable[0] != self.executable[1])
+        return False
+
+    def is_reparented(self):
+        return self.parent_id[0] != self.parent_id[1]
+
+    @property
+    def renamed(self):
+        return (
+            not self.copied and
+            None not in self.name and
+            None not in self.parent_id and
+            (self.name[0] != self.name[1] or self.parent_id[0] != self.parent_id[1]))
+
+    def discard_new(self):
+        return self.__class__(
+            self.file_id, (self.path[0], None), self.changed_content,
+            (self.versioned[0], None), (self.parent_id[0], None),
+            (self.name[0], None), (self.kind[0], None),
+            (self.executable[0], None),
+            copied=False)
+
+
 class InventoryTree(Tree):
     """A tree that relies on an inventory for its metadata.
 
@@ -79,12 +133,21 @@ class InventoryTree(Tree):
     private to external API users.
     """
 
+    def supports_symlinks(self):
+        return True
+
+    @classmethod
+    def is_special_path(cls, path):
+        return path.startswith('.bzr')
+
     def _get_root_inventory(self):
         return self._inventory
 
     root_inventory = property(_get_root_inventory,
                               doc="Root inventory of this tree")
 
+    supports_file_ids = True
+
     def _unpack_file_id(self, file_id):
         """Find the inventory and inventory file id for a tree file id.
 
@@ -150,6 +213,10 @@ class InventoryTree(Tree):
         with self.lock_read():
             return self._path2inv_file_id(path)[1]
 
+
+    def is_versioned(self, path):
+        return self.path2id(path) is not None
+
     def _path2ie(self, path):
         """Lookup an inventory entry by path.
 
@@ -158,7 +225,7 @@ class InventoryTree(Tree):
         """
         inv, ie = self._path2inv_ie(path)
         if ie is None:
-            raise errors.NoSuchFile(path)
+            raise _mod_transport.NoSuchFile(path)
         return ie
 
     def _path2inv_ie(self, path):
@@ -237,9 +304,15 @@ class InventoryTree(Tree):
             def iter_entries(inv):
                 for p, e in inv.iter_entries_by_dir(specific_file_ids=inventory_file_ids):
                     if e.kind == 'tree-reference' and recurse_nested:
-                        subinv = self._get_nested_tree(p, e.file_id, e.reference_revision).root_inventory
-                        for subp, e in iter_entries(subinv):
-                            yield (osutils.pathjoin(p, subp) if subp else p), e
+                        try:
+                            subtree = self._get_nested_tree(p, e.file_id, e.reference_revision)
+                        except errors.NotBranchError:
+                            yield p, e
+                        else:
+                            with subtree.lock_read():
+                                subinv = subtree.root_inventory
+                                for subp, e in iter_entries(subinv):
+                                    yield (osutils.pathjoin(p, subp) if subp else p), e
                     else:
                         yield p, e
             return iter_entries(self.root_inventory)
@@ -249,7 +322,7 @@ class InventoryTree(Tree):
             ie = self._path2ie(path)
             if ie.kind != 'directory':
                 raise errors.NotADirectory(path)
-            return iter(viewvalues(ie.children))
+            return ie.children.values()
 
     def _get_plan_merge_data(self, path, other, base):
         from . import versionedfile
@@ -317,6 +390,10 @@ class InventoryTree(Tree):
             vf.fallback_versionedfiles.append(base_vf)
         return last_revision
 
+    def preview_transform(self, pb=None):
+        from .transform import TransformPreview
+        return TransformPreview(self, pb=pb)
+
 
 def find_ids_across_trees(filenames, trees, require_versioned=True):
     """Find the ids corresponding to specified filenames.
@@ -412,6 +489,51 @@ class MutableInventoryTree(MutableTree, InventoryTree):
             inv.apply_delta(changes)
             self._write_inventory(inv)
 
+    def has_changes(self, _from_tree=None):
+        """Quickly check that the tree contains at least one commitable change.
+
+        :param _from_tree: tree to compare against to find changes (default to
+            the basis tree and is intended to be used by tests).
+
+        :return: True if a change is found. False otherwise
+        """
+        with self.lock_read():
+            # Check pending merges
+            if len(self.get_parent_ids()) > 1:
+                return True
+            if _from_tree is None:
+                _from_tree = self.basis_tree()
+            changes = self.iter_changes(_from_tree)
+            if self.supports_symlinks():
+                # Fast path for has_changes.
+                try:
+                    change = next(changes)
+                    # Exclude root (talk about black magic... --vila 20090629)
+                    if change.parent_id == (None, None):
+                        change = next(changes)
+                    return True
+                except StopIteration:
+                    # No changes
+                    return False
+            else:
+                # Slow path for has_changes.
+                # Handle platforms that do not support symlinks in the
+                # conditional below. This is slower than the try/except
+                # approach below that but we don't have a choice as we
+                # need to be sure that all symlinks are removed from the
+                # entire changeset. This is because in platforms that
+                # do not support symlinks, they show up as None in the
+                # working copy as compared to the repository.
+                # Also, exclude root as mention in the above fast path.
+                changes = filter(
+                    lambda c: c[6][0] != 'symlink' and c[4] != (None, None),
+                    changes)
+                try:
+                    next(iter(changes))
+                except StopIteration:
+                    return False
+                return True
+
     def _fix_case_of_inventory_path(self, path):
         """If our tree isn't case sensitive, return the canonical path"""
         if not self.case_sensitive:
@@ -498,9 +620,78 @@ class MutableInventoryTree(MutableTree, InventoryTree):
                                          inventory, new_revid)
         self.set_parent_trees([(new_revid, rev_tree)])
 
-    def get_transform(self, pb=None):
-        from ..transform import TreeTransform
-        return TreeTransform(self, pb=pb)
+    def transform(self, pb=None):
+        from .transform import InventoryTreeTransform
+        return InventoryTreeTransform(self, pb=pb)
+
+    def add(self, files, kinds=None, ids=None):
+        """Add paths to the set of versioned paths.
+
+        Note that the command line normally calls smart_add instead,
+        which can automatically recurse.
+
+        This adds the files to the tree, so that they will be
+        recorded by the next commit.
+
+        Args:
+          files: List of paths to add, relative to the base of the tree.
+          kinds: Optional parameter to specify the kinds to be used for
+            each file.
+          ids: If set, use these instead of automatically generated ids.
+            Must be the same length as the list of files, but may
+            contain None for ids that are to be autogenerated.
+
+        TODO: Perhaps callback with the ids and paths as they're added.
+        """
+        if isinstance(files, str):
+            # XXX: Passing a single string is inconsistent and should be
+            # deprecated.
+            if not (ids is None or isinstance(ids, bytes)):
+                raise AssertionError()
+            if not (kinds is None or isinstance(kinds, str)):
+                raise AssertionError()
+            files = [files]
+            if ids is not None:
+                ids = [ids]
+            if kinds is not None:
+                kinds = [kinds]
+
+        files = [path.strip('/') for path in files]
+
+        if ids is None:
+            ids = [None] * len(files)
+        else:
+            if not (len(ids) == len(files)):
+                raise AssertionError()
+        if kinds is None:
+            kinds = [None] * len(files)
+        elif not len(kinds) == len(files):
+            raise AssertionError()
+        with self.lock_tree_write():
+            for f in files:
+                # generic constraint checks:
+                if self.is_control_filename(f):
+                    raise errors.ForbiddenControlFileError(filename=f)
+                fp = osutils.splitpath(f)
+            # fill out file kinds for all files [not needed when we stop
+            # caring about the instantaneous file kind within a uncommmitted tree
+            #
+            self._gather_kinds(files, kinds)
+            self._add(files, kinds, ids)
+
+    def _gather_kinds(self, files, kinds):
+        """Helper function for add - sets the entries of kinds."""
+        raise NotImplementedError(self._gather_kinds)
+
+    def _add(self, files, kinds, ids):
+        """Helper function for add - updates the inventory.
+
+        :param files: sequence of pathnames, relative to the tree root
+        :param kinds: sequence of  inventory kinds of the files (i.e. may
+            contain "tree-reference")
+        :param ids: sequence of suggested ids for the files (may be None)
+        """
+        raise NotImplementedError(self._add)
 
 
 class _SmartAddHelper(object):
@@ -509,7 +700,7 @@ class _SmartAddHelper(object):
     def get_inventory_delta(self):
         # GZ 2016-06-05: Returning view would probably be fine but currently
         # Inventory.apply_delta is documented as requiring a list of changes.
-        return list(viewvalues(self._invdelta))
+        return list(self._invdelta.values())
 
     def _get_ie(self, inv_path):
         """Retrieve the most up to date inventory entry for a path.
@@ -856,8 +1047,11 @@ class InventoryRevisionTree(RevisionTree, InventoryTree):
 
     def _get_nested_tree(self, path, file_id, reference_revision):
         # Just a guess..
-        subdir = ControlDir.open_from_transport(
-            self._repository.user_transport.clone(path))
+        try:
+            subdir = ControlDir.open_from_transport(
+                self._repository.user_transport.clone(path))
+        except errors.NotBranchError as e:
+            raise MissingNestedTree(path) from e
         subrepo = subdir.find_repository()
         try:
             revtree = subrepo.revision_tree(reference_revision)
@@ -879,7 +1073,7 @@ class InventoryRevisionTree(RevisionTree, InventoryTree):
         """See Tree.path_content_summary."""
         try:
             entry = self._path2ie(path)
-        except errors.NoSuchFile:
+        except _mod_transport.NoSuchFile:
             return ('missing', None, None, None)
         kind = entry.kind
         if kind == 'file':
@@ -900,27 +1094,25 @@ class InventoryRevisionTree(RevisionTree, InventoryTree):
         if top_id is None:
             pending = []
         else:
-            pending = [(prefix, '', _directory, None, top_id, None)]
+            pending = [(prefix, top_id)]
         while pending:
             dirblock = []
-            currentdir = pending.pop()
-            # 0 - relpath, 1- basename, 2- kind, 3- stat, id, v-kind
-            if currentdir[0]:
-                relroot = currentdir[0] + '/'
+            root, file_id = pending.pop()
+            if root:
+                relroot = root + '/'
             else:
                 relroot = ""
             # FIXME: stash the node in pending
-            entry = inv.get_entry(currentdir[4])
+            entry = inv.get_entry(file_id)
+            subdirs = []
             for name, child in entry.sorted_children():
                 toppath = relroot + name
-                dirblock.append((toppath, name, child.kind, None,
-                                 child.file_id, child.kind
-                                 ))
-            yield (currentdir[0], entry.file_id), dirblock
+                dirblock.append((toppath, name, child.kind, None, child.kind))
+                if child.kind == _directory:
+                    subdirs.append((toppath, child.file_id))
+            yield root, dirblock
             # push the user specified dirs from dirblock
-            for dir in reversed(dirblock):
-                if dir[2] == _directory:
-                    pending.append(dir)
+            pending.extend(reversed(subdirs))
 
     def iter_files_bytes(self, desired_files):
         """See Tree.iter_files_bytes.
@@ -932,7 +1124,7 @@ class InventoryRevisionTree(RevisionTree, InventoryTree):
             for result in self._repository.iter_files_bytes(repo_desired_files):
                 yield result
         except errors.RevisionNotPresent as e:
-            raise errors.NoSuchFile(e.file_id)
+            raise _mod_transport.NoSuchFile(e.file_id)
 
     def annotate_iter(self, path, default_revision=revision.CURRENT_REVISION):
         """See Tree.annotate_iter"""
@@ -956,7 +1148,392 @@ class InventoryRevisionTree(RevisionTree, InventoryTree):
         raise ValueError('not hashable')
 
 
-class InterCHKRevisionTree(InterTree):
+class InterInventoryTree(InterTree):
+    """InterTree implementation for InventoryTree objects.
+
+    """
+    @classmethod
+    def is_compatible(kls, source, target):
+        # The default implementation is naive and uses the public API, so
+        # it works for all trees.
+        return (isinstance(source, InventoryTree) and
+                isinstance(target, InventoryTree))
+
+    def _changes_from_entries(self, source_entry, target_entry, source_path,
+                              target_path):
+        """Generate a iter_changes tuple between source_entry and target_entry.
+
+        :param source_entry: An inventory entry from self.source, or None.
+        :param target_entry: An inventory entry from self.target, or None.
+        :param source_path: The path of source_entry.
+        :param target_path: The path of target_entry.
+        :return: A tuple, item 0 of which is an iter_changes result tuple, and
+            item 1 is True if there are any changes in the result tuple.
+        """
+        if source_entry is None:
+            if target_entry is None:
+                return None
+            file_id = target_entry.file_id
+        else:
+            file_id = source_entry.file_id
+        if source_entry is not None:
+            source_versioned = True
+            source_name = source_entry.name
+            source_parent = source_entry.parent_id
+            source_kind, source_executable, source_stat = \
+                self.source._comparison_data(source_entry, source_path)
+        else:
+            source_versioned = False
+            source_name = None
+            source_parent = None
+            source_kind = None
+            source_executable = None
+        if target_entry is not None:
+            target_versioned = True
+            target_name = target_entry.name
+            target_parent = target_entry.parent_id
+            target_kind, target_executable, target_stat = \
+                self.target._comparison_data(target_entry, target_path)
+        else:
+            target_versioned = False
+            target_name = None
+            target_parent = None
+            target_kind = None
+            target_executable = None
+        versioned = (source_versioned, target_versioned)
+        kind = (source_kind, target_kind)
+        changed_content = False
+        if source_kind != target_kind:
+            changed_content = True
+        elif source_kind == 'file':
+            if not self.file_content_matches(
+                    source_path, target_path,
+                    source_stat, target_stat):
+                changed_content = True
+        elif source_kind == 'symlink':
+            if (self.source.get_symlink_target(source_path) !=
+                    self.target.get_symlink_target(target_path)):
+                changed_content = True
+        elif source_kind == 'tree-reference':
+            if (self.source.get_reference_revision(source_path)
+                    != self.target.get_reference_revision(target_path)):
+                changed_content = True
+        parent = (source_parent, target_parent)
+        name = (source_name, target_name)
+        executable = (source_executable, target_executable)
+        if (changed_content is not False or versioned[0] != versioned[1] or
+            parent[0] != parent[1] or name[0] != name[1] or
+                executable[0] != executable[1]):
+            changes = True
+        else:
+            changes = False
+        return InventoryTreeChange(
+            file_id, (source_path, target_path), changed_content,
+            versioned, parent, name, kind, executable), changes
+
+    def iter_changes(self, include_unchanged=False,
+                     specific_files=None, pb=None, extra_trees=[],
+                     require_versioned=True, want_unversioned=False):
+        """Generate an iterator of changes between trees.
+
+        A tuple is returned:
+        (file_id, (path_in_source, path_in_target),
+         changed_content, versioned, parent, name, kind,
+         executable)
+
+        Changed_content is True if the file's content has changed.  This
+        includes changes to its kind, and to a symlink's target.
+
+        versioned, parent, name, kind, executable are tuples of (from, to).
+        If a file is missing in a tree, its kind is None.
+
+        Iteration is done in parent-to-child order, relative to the target
+        tree.
+
+        There is no guarantee that all paths are in sorted order: the
+        requirement to expand the search due to renames may result in children
+        that should be found early being found late in the search, after
+        lexically later results have been returned.
+        :param require_versioned: Raise errors.PathsNotVersionedError if a
+            path in the specific_files list is not versioned in one of
+            source, target or extra_trees.
+        :param specific_files: An optional list of file paths to restrict the
+            comparison to. When mapping filenames to ids, all matches in all
+            trees (including optional extra_trees) are used, and all children
+            of matched directories are included. The parents in the target tree
+            of the specific files up to and including the root of the tree are
+            always evaluated for changes too.
+        :param want_unversioned: Should unversioned files be returned in the
+            output. An unversioned file is defined as one with (False, False)
+            for the versioned pair.
+        """
+        if not extra_trees:
+            extra_trees = []
+        else:
+            extra_trees = list(extra_trees)
+        # The ids of items we need to examine to insure delta consistency.
+        precise_file_ids = set()
+        changed_file_ids = []
+        if specific_files == []:
+            target_specific_files = []
+            source_specific_files = []
+        else:
+            target_specific_files = self.target.find_related_paths_across_trees(
+                specific_files, [self.source] + extra_trees,
+                require_versioned=require_versioned)
+            source_specific_files = self.source.find_related_paths_across_trees(
+                specific_files, [self.target] + extra_trees,
+                require_versioned=require_versioned)
+        if specific_files is not None:
+            # reparented or added entries must have their parents included
+            # so that valid deltas can be created. The seen_parents set
+            # tracks the parents that we need to have.
+            # The seen_dirs set tracks directory entries we've yielded.
+            # After outputting version object in to_entries we set difference
+            # the two seen sets and start checking parents.
+            seen_parents = set()
+            seen_dirs = set()
+        if want_unversioned:
+            all_unversioned = sorted([(p.split('/'), p) for p in
+                                      self.target.extras()
+                                      if specific_files is None or
+                                      osutils.is_inside_any(specific_files, p)])
+            all_unversioned = deque(all_unversioned)
+        else:
+            all_unversioned = deque()
+        to_paths = {}
+        from_entries_by_dir = list(self.source.iter_entries_by_dir(
+            specific_files=source_specific_files))
+        from_data = dict(from_entries_by_dir)
+        to_entries_by_dir = list(self.target.iter_entries_by_dir(
+            specific_files=target_specific_files))
+        path_equivs = self.find_source_paths([p for p, e in to_entries_by_dir])
+        num_entries = len(from_entries_by_dir) + len(to_entries_by_dir)
+        entry_count = 0
+        # the unversioned path lookup only occurs on real trees - where there
+        # can be extras. So the fake_entry is solely used to look up
+        # executable it values when execute is not supported.
+        fake_entry = TreeFile()
+        for target_path, target_entry in to_entries_by_dir:
+            while (all_unversioned and
+                   all_unversioned[0][0] < target_path.split('/')):
+                unversioned_path = all_unversioned.popleft()
+                target_kind, target_executable, target_stat = \
+                    self.target._comparison_data(
+                        fake_entry, unversioned_path[1])
+                yield InventoryTreeChange(
+                    None, (None, unversioned_path[1]), True, (False, False),
+                    (None, None),
+                    (None, unversioned_path[0][-1]),
+                    (None, target_kind),
+                    (None, target_executable))
+            source_path = path_equivs[target_path]
+            if source_path is not None:
+                source_entry = from_data.get(source_path)
+            else:
+                source_entry = None
+            result, changes = self._changes_from_entries(
+                source_entry, target_entry, source_path=source_path, target_path=target_path)
+            to_paths[result.file_id] = result.path[1]
+            entry_count += 1
+            if result.versioned[0]:
+                entry_count += 1
+            if pb is not None:
+                pb.update('comparing files', entry_count, num_entries)
+            if changes or include_unchanged:
+                if specific_files is not None:
+                    precise_file_ids.add(result.parent_id[1])
+                    changed_file_ids.append(result.file_id)
+                yield result
+            # Ensure correct behaviour for reparented/added specific files.
+            if specific_files is not None:
+                # Record output dirs
+                if result.kind[1] == 'directory':
+                    seen_dirs.add(result.file_id)
+                # Record parents of reparented/added entries.
+                if not result.versioned[0] or result.is_reparented():
+                    seen_parents.add(result.parent_id[1])
+        while all_unversioned:
+            # yield any trailing unversioned paths
+            unversioned_path = all_unversioned.popleft()
+            to_kind, to_executable, to_stat = \
+                self.target._comparison_data(fake_entry, unversioned_path[1])
+            yield InventoryTreeChange(
+                None, (None, unversioned_path[1]), True, (False, False),
+                (None, None),
+                (None, unversioned_path[0][-1]),
+                (None, to_kind),
+                (None, to_executable))
+        # Yield all remaining source paths
+        for path, from_entry in from_entries_by_dir:
+            file_id = from_entry.file_id
+            if file_id in to_paths:
+                # already returned
+                continue
+            to_path = self.find_target_path(path)
+            entry_count += 1
+            if pb is not None:
+                pb.update('comparing files', entry_count, num_entries)
+            versioned = (True, False)
+            parent = (from_entry.parent_id, None)
+            name = (from_entry.name, None)
+            from_kind, from_executable, stat_value = \
+                self.source._comparison_data(from_entry, path)
+            kind = (from_kind, None)
+            executable = (from_executable, None)
+            changed_content = from_kind is not None
+            # the parent's path is necessarily known at this point.
+            changed_file_ids.append(file_id)
+            yield InventoryTreeChange(
+                file_id, (path, to_path), changed_content, versioned, parent,
+                name, kind, executable)
+        changed_file_ids = set(changed_file_ids)
+        if specific_files is not None:
+            for result in self._handle_precise_ids(precise_file_ids,
+                                                   changed_file_ids):
+                yield result
+
+    @staticmethod
+    def _get_entry(tree, path):
+        """Get an inventory entry from a tree, with missing entries as None.
+
+        If the tree raises NotImplementedError on accessing .inventory, then
+        this is worked around using iter_entries_by_dir on just the file id
+        desired.
+
+        :param tree: The tree to lookup the entry in.
+        :param path: The path to look up
+        """
+        # No inventory available.
+        try:
+            iterator = tree.iter_entries_by_dir(specific_files=[path])
+            return next(iterator)[1]
+        except StopIteration:
+            return None
+
+    def _handle_precise_ids(self, precise_file_ids, changed_file_ids,
+                            discarded_changes=None):
+        """Fill out a partial iter_changes to be consistent.
+
+        :param precise_file_ids: The file ids of parents that were seen during
+            the iter_changes.
+        :param changed_file_ids: The file ids of already emitted items.
+        :param discarded_changes: An optional dict of precalculated
+            iter_changes items which the partial iter_changes had not output
+            but had calculated.
+        :return: A generator of iter_changes items to output.
+        """
+        # process parents of things that had changed under the users
+        # requested paths to prevent incorrect paths or parent ids which
+        # aren't in the tree.
+        while precise_file_ids:
+            precise_file_ids.discard(None)
+            # Don't emit file_ids twice
+            precise_file_ids.difference_update(changed_file_ids)
+            if not precise_file_ids:
+                break
+            # If the there was something at a given output path in source, we
+            # have to include the entry from source in the delta, or we would
+            # be putting this entry into a used path.
+            paths = []
+            for parent_id in precise_file_ids:
+                try:
+                    paths.append(self.target.id2path(parent_id))
+                except errors.NoSuchId:
+                    # This id has been dragged in from the source by delta
+                    # expansion and isn't present in target at all: we don't
+                    # need to check for path collisions on it.
+                    pass
+            for path in paths:
+                old_id = self.source.path2id(path)
+                precise_file_ids.add(old_id)
+            precise_file_ids.discard(None)
+            current_ids = precise_file_ids
+            precise_file_ids = set()
+            # We have to emit all of precise_file_ids that have been altered.
+            # We may have to output the children of some of those ids if any
+            # directories have stopped being directories.
+            for file_id in current_ids:
+                # Examine file_id
+                if discarded_changes:
+                    result = discarded_changes.get(file_id)
+                    source_entry = None
+                else:
+                    result = None
+                if result is None:
+                    try:
+                        source_path = self.source.id2path(file_id)
+                    except errors.NoSuchId:
+                        source_path = None
+                        source_entry = None
+                    else:
+                        source_entry = self._get_entry(
+                            self.source, source_path)
+                    try:
+                        target_path = self.target.id2path(file_id)
+                    except errors.NoSuchId:
+                        target_path = None
+                        target_entry = None
+                    else:
+                        target_entry = self._get_entry(
+                            self.target, target_path)
+                    result, changes = self._changes_from_entries(
+                        source_entry, target_entry, source_path, target_path)
+                else:
+                    changes = True
+                # Get this parents parent to examine.
+                new_parent_id = result.parent_id[1]
+                precise_file_ids.add(new_parent_id)
+                if changes:
+                    if (result.kind[0] == 'directory' and
+                            result.kind[1] != 'directory'):
+                        # This stopped being a directory, the old children have
+                        # to be included.
+                        if source_entry is None:
+                            # Reusing a discarded change.
+                            source_entry = self._get_entry(
+                                self.source, result.path[0])
+                        precise_file_ids.update(
+                            child.file_id
+                            for child in self.source.iter_child_entries(result.path[0]))
+                    changed_file_ids.add(result.file_id)
+                    yield result
+
+    def find_target_path(self, path, recurse='none'):
+        """Find target tree path.
+
+        :param path: Path to search for (exists in source)
+        :return: path in target, or None if there is no equivalent path.
+        :raise NoSuchFile: If the path doesn't exist in source
+        """
+        file_id = self.source.path2id(path)
+        if file_id is None:
+            raise _mod_transport.NoSuchFile(path)
+        try:
+            return self.target.id2path(file_id, recurse=recurse)
+        except errors.NoSuchId:
+            return None
+
+    def find_source_path(self, path, recurse='none'):
+        """Find the source tree path.
+
+        :param path: Path to search for (exists in target)
+        :return: path in source, or None if there is no equivalent path.
+        :raise NoSuchFile: if the path doesn't exist in target
+        """
+        file_id = self.target.path2id(path)
+        if file_id is None:
+            raise _mod_transport.NoSuchFile(path)
+        try:
+            return self.source.id2path(file_id, recurse=recurse)
+        except errors.NoSuchId:
+            return None
+
+
+InterTree.register_optimiser(InterInventoryTree)
+
+
+class InterCHKRevisionTree(InterInventoryTree):
     """Fast path optimiser for RevisionTrees with CHK inventories."""
 
     @staticmethod
@@ -993,7 +1570,7 @@ class InterCHKRevisionTree(InterTree):
         # FIXME: nested tree support
         for result in self.target.root_inventory.iter_changes(
                 self.source.root_inventory):
-            result = TreeChange(*result)
+            result = InventoryTreeChange(*result)
             if specific_file_ids is not None:
                 if result.file_id not in specific_file_ids:
                     # A change from the whole tree that we don't want to show yet.
@@ -1020,7 +1597,7 @@ class InterCHKRevisionTree(InterTree):
                         entry.file_id not in specific_file_ids):
                     continue
                 if entry.file_id not in changed_file_ids:
-                    yield TreeChange(
+                    yield InventoryTreeChange(
                         entry.file_id,
                         (relpath, relpath),  # Not renamed
                         False,  # Not modified
