@@ -19,10 +19,7 @@ client and server.
 """
 
 
-try:
-    from collections.abc import deque
-except ImportError:  # python < 3.7
-    from collections import deque
+from collections import deque
 
 from io import BytesIO
 import struct
