@@ -26,18 +26,13 @@ Interesting module attributes:
       of times during a request).
 """
 
-from __future__ import absolute_import
-
 # XXX: The class names are a little confusing: the protocol will instantiate a
 # SmartServerRequestHandler, whose dispatch_command method creates an instance
 # of a SmartServerRequest subclass.
 
 
 import threading
-try:
-    from _thread import get_ident
-except ImportError:  # Python < 3
-    from thread import get_ident
+from _thread import get_ident
 
 from ... import (
     branch as _mod_branch,
@@ -47,13 +42,13 @@ from ... import (
     registry,
     revision,
     trace,
+    transport as _mod_transport,
     urlutils,
     )
-from ...sixish import text_type
 from ...lazy_import import lazy_import
 lazy_import(globals(), """
 from breezy.bzr import bzrdir
-from breezy.bundle import serializer
+from breezy.bzr.bundle import serializer
 
 import tempfile
 """)
@@ -419,9 +414,9 @@ class SmartServerRequestHandler(object):
 
 
 def _translate_error(err):
-    if isinstance(err, errors.NoSuchFile):
+    if isinstance(err, _mod_transport.NoSuchFile):
         return (b'NoSuchFile', err.path.encode('utf-8'))
-    elif isinstance(err, errors.FileExists):
+    elif isinstance(err, _mod_transport.FileExists):
         return (b'FileExists', err.path.encode('utf-8'))
     elif isinstance(err, errors.DirectoryNotEmpty):
         return (b'DirectoryNotEmpty', err.path.encode('utf-8'))
@@ -449,7 +444,7 @@ def _translate_error(err):
         # If it is a DecodeError, than most likely we are starting
         # with a plain string
         str_or_unicode = err.object
-        if isinstance(str_or_unicode, text_type):
+        if isinstance(str_or_unicode, str):
             # XXX: UTF-8 might have \x01 (our protocol v1 and v2 seperator
             # byte) in it, so this encoding could cause broken responses.
             # Newer clients use protocol v3, so will be fine.
@@ -478,6 +473,8 @@ def _translate_error(err):
     elif isinstance(err, MemoryError):
         # GZ 2011-02-24: Copy breezy.trace -Dmem_dump functionality here?
         return (b'MemoryError',)
+    elif isinstance(err, errors.AlreadyControlDirError):
+        return (b'AlreadyControlDir', err.path)
     # Unserialisable error.  Log it, and return a generic error
     trace.log_exception_quietly()
     return (b'error',
@@ -545,7 +542,7 @@ class SmartServerIsReadonly(SmartServerRequest):
 #   mutate  State is updated in a way that replaying that request results in a
 #           different state. For example 'append' writes more bytes to a given
 #           file. If append succeeds, it moves the file pointer.
-request_handlers = registry.Registry()
+request_handlers = registry.Registry[bytes, SmartServerRequest]()
 request_handlers.register_lazy(
     b'append', 'breezy.bzr.smart.vfs', 'AppendRequest', info='mutate')
 request_handlers.register_lazy(
@@ -608,6 +605,9 @@ request_handlers.register_lazy(
 request_handlers.register_lazy(
     b'Branch.revision_id_to_revno', 'breezy.bzr.smart.branch',
     'SmartServerBranchRequestRevisionIdToRevno', info='read')
+request_handlers.register_lazy(
+    b'Branch.get_all_reference_info', 'breezy.bzr.smart.branch',
+    'SmartServerBranchRequestGetAllReferenceInfo', info='read')
 request_handlers.register_lazy(
     b'BzrDir.checkout_metadir', 'breezy.bzr.smart.bzrdir',
     'SmartServerBzrDirRequestCheckoutMetaDir', info='read')
