@@ -24,14 +24,14 @@ see breezy.bzrdir.BzrDir.
 
 """
 
+from typing import List, Type, Optional
+
 from .lazy_import import lazy_import
 lazy_import(globals(), """
 import textwrap
 
 from breezy import (
     branch as _mod_branch,
-    hooks,
-    revision as _mod_revision,
     ui,
     urlutils,
     )
@@ -42,7 +42,9 @@ from breezy.i18n import gettext
 
 from . import (
     errors,
+    hooks,
     registry,
+    revision as _mod_revision,
     trace,
     transport as _mod_transport,
     )
@@ -125,6 +127,8 @@ class ControlDir(ControlComponent):
     API users to check for magic attributes to see what features are supported.
     """
 
+    hooks: hooks.Hooks
+
     def can_convert_format(self):
         """Return true if this controldir is one whose format we can convert
         from."""
@@ -948,7 +952,7 @@ class ControlDirHooks(hooks.Hooks):
 
 
 # install the default hooks
-ControlDir.hooks = ControlDirHooks()
+ControlDir.hooks = ControlDirHooks()  # type: ignore
 
 
 class ControlComponentFormat(object):
@@ -994,7 +998,7 @@ class ControlComponentFormat(object):
         raise NotImplementedError(cls.get_format_string)
 
 
-class ControlComponentFormatRegistry(registry.FormatRegistry):
+class ControlComponentFormatRegistry(registry.FormatRegistry[ControlComponentFormat]):
     """A registry for control components (branch, workingtree, repository)."""
 
     def __init__(self, other_registry=None):
@@ -1099,10 +1103,10 @@ class ControlDirFormat(object):
                                  working tree.
     """
 
-    _default_format = None
+    _default_format: Optional["ControlDirFormat"] = None
     """The default format used for new control directories."""
 
-    _probers = []
+    _probers: List[Type["Prober"]] = []
     """The registered format probers, e.g. BzrProber.
 
     This is a list of Prober-derived classes.
@@ -1181,14 +1185,14 @@ class ControlDirFormat(object):
                 target_format.rich_root_data)
 
     @classmethod
-    def register_prober(klass, prober):
+    def register_prober(klass, prober: Type["Prober"]):
         """Register a prober that can look for a control dir.
 
         """
         klass._probers.append(prober)
 
     @classmethod
-    def unregister_prober(klass, prober):
+    def unregister_prober(klass, prober: Type["Prober"]):
         """Unregister a prober.
 
         """
@@ -1199,7 +1203,7 @@ class ControlDirFormat(object):
         return self.get_format_description().rstrip()
 
     @classmethod
-    def all_probers(klass):
+    def all_probers(klass) -> List[Type["Prober"]]:
         return klass._probers
 
     @classmethod
@@ -1325,7 +1329,7 @@ class ControlDirFormat(object):
         this in the future - for instance to make bzr talk with svn working
         trees.
         """
-        raise NotImplementedError(self.is_control_filename)
+        raise NotImplementedError(cls.is_control_filename)
 
 
 class Prober(object):
@@ -1390,7 +1394,7 @@ class ControlDirFormatInfo(object):
         self.experimental = experimental
 
 
-class ControlDirFormatRegistry(registry.Registry):
+class ControlDirFormatRegistry(registry.Registry[str, ControlDirFormat]):
     """Registry of user-selectable ControlDir subformats.
 
     Differs from ControlDirFormat._formats in that it provides sub-formats,
@@ -1673,7 +1677,7 @@ class RepositoryAcquisitionPolicy(object):
 # on previous ones.
 format_registry = ControlDirFormatRegistry()
 
-network_format_registry = registry.FormatRegistry()
+network_format_registry = registry.FormatRegistry[ControlDirFormat]()
 """Registry of formats indexed by their network name.
 
 The network name for a ControlDirFormat is an identifier that can be used when
