@@ -19,6 +19,7 @@ import os
 import subprocess
 import sys
 import tempfile
+from typing import Type
 
 import breezy
 from . import (
@@ -31,9 +32,6 @@ from . import (
     registry,
     )
 
-mail_client_registry = registry.Registry()
-
-
 class MailClientNotFound(errors.BzrError):
 
     _fmt = "Unable to find mail client with the following names:"\
@@ -110,6 +108,9 @@ class MailClient(object):
         return ''
 
 
+mail_client_registry = registry.Registry[str, Type[MailClient]]()
+
+
 class Editor(MailClient):
     __doc__ = """DIY mail client that uses commit message editor"""
 
