@@ -36,7 +36,7 @@ from ...option import (
     )
 from ...trace import note, warning
 from ... import (
-    propose as _mod_propose,
+    forge as _mod_forge,
     )
 
 
@@ -89,8 +89,8 @@ class cmd_publish_derived(Command):
         _check_already_merged(local_branch, submit_branch)
         if name is None:
             name = branch_name(local_branch)
-        hoster = _mod_propose.get_hoster(submit_branch)
-        remote_branch, public_url = hoster.publish_derived(
+        forge = _mod_forge.get_forge(submit_branch)
+        remote_branch, public_url = forge.publish_derived(
             local_branch, submit_branch, name=name, project=project,
             owner=owner, allow_lossy=not no_allow_lossy,
             overwrite=overwrite)
@@ -141,9 +141,9 @@ class cmd_propose_merge(Command):
     takes_options = [
         'directory',
         RegistryOption(
-            'hoster',
-            help='Use the hoster.',
-            lazy_registry=('breezy.propose', 'hosters')),
+            'forge',
+            help='Use the forge.',
+            lazy_registry=('breezy.forge', 'forges')),
         ListOption('reviewers', short_name='R', type=str,
                    help='Requested reviewers.'),
         Option('name', help='Name of the new remote branch.', type=str),
@@ -162,15 +162,17 @@ class cmd_propose_merge(Command):
         Option('allow-empty',
                help='Do not prevent empty merge proposals.'),
         Option('overwrite', help="Overwrite existing commits."),
+        Option('open', help='Open merge proposal in web browser'),
         ]
     takes_args = ['submit_branch?']
 
     aliases = ['propose']
 
-    def run(self, submit_branch=None, directory='.', hoster=None,
+    def run(self, submit_branch=None, directory='.', forge=None,
             reviewers=None, name=None, no_allow_lossy=False, description=None,
             labels=None, prerequisite=None, commit_message=None, wip=False,
-            allow_collaboration=False, allow_empty=False, overwrite=False):
+            allow_collaboration=False, allow_empty=False, overwrite=False,
+            open=False):
         tree, branch, relpath = (
             controldir.ControlDir.open_containing_tree_or_branch(directory))
         if submit_branch is None:
@@ -183,23 +185,24 @@ class cmd_propose_merge(Command):
         target = _mod_branch.Branch.open(submit_branch)
         if not allow_empty:
             _check_already_merged(branch, target)
-        if hoster is None:
-            hoster = _mod_propose.get_hoster(target)
+        if forge is None:
+            forge = _mod_forge.get_forge(target)
         else:
-            hoster = hoster.probe(target)
+            forge = forge.probe(target)
         if name is None:
             name = branch_name(branch)
-        remote_branch, public_branch_url = hoster.publish_derived(
+        remote_branch, public_branch_url = forge.publish_derived(
             branch, target, name=name, allow_lossy=not no_allow_lossy,
             overwrite=overwrite)
         branch.set_push_location(remote_branch.user_url)
         branch.set_submit_branch(target.user_url)
-        note(gettext('Published branch to %s') % public_branch_url)
+        note(gettext('Published branch to %s'),
+             forge.get_web_url(remote_branch) or public_branch_url)
         if prerequisite is not None:
             prerequisite_branch = _mod_branch.Branch.open(prerequisite)
         else:
             prerequisite_branch = None
-        proposal_builder = hoster.get_proposer(remote_branch, target)
+        proposal_builder = forge.get_proposer(remote_branch, target)
         if description is None:
             body = proposal_builder.get_initial_body()
             info = proposal_builder.get_infotext()
@@ -213,10 +216,15 @@ class cmd_propose_merge(Command):
                 prerequisite_branch=prerequisite_branch, labels=labels,
                 commit_message=commit_message,
                 work_in_progress=wip, allow_collaboration=allow_collaboration)
-        except _mod_propose.MergeProposalExists as e:
+        except _mod_forge.MergeProposalExists as e:
             note(gettext('There is already a branch merge proposal: %s'), e.url)
         else:
             note(gettext('Merge proposal created: %s') % proposal.url)
+            web_url = proposal.get_web_url()
+            note(gettext('Opening %s in web browser'), web_url)
+            if open:
+                import webbrowser
+                webbrowser.open(web_url)
 
 
 class cmd_find_merge_proposal(Command):
@@ -243,8 +251,8 @@ class cmd_find_merge_proposal(Command):
                 gettext("No target location specified or remembered"))
         else:
             target = _mod_branch.Branch.open(submit_branch)
-        hoster = _mod_propose.get_hoster(branch)
-        for mp in hoster.iter_proposals(branch, target):
+        forge = _mod_forge.get_forge(branch)
+        for mp in forge.iter_proposals(branch, target):
             self.outf.write(gettext('Merge proposal: %s\n') % mp.url)
 
 
@@ -269,14 +277,14 @@ class cmd_my_merge_proposals(Command):
             merged='Merged merge proposals',
             closed='Closed merge proposals'),
         RegistryOption(
-            'hoster',
-            help='Use the hoster.',
-            lazy_registry=('breezy.propose', 'hosters')),
+            'forge',
+            help='Use the forge.',
+            lazy_registry=('breezy.forge', 'forges')),
         ]
 
-    def run(self, status='open', verbose=False, hoster=None, base_url=None):
+    def run(self, status='open', verbose=False, forge=None, base_url=None):
 
-        for instance in _mod_propose.iter_hoster_instances(hoster=hoster):
+        for instance in _mod_forge.iter_forge_instances(forge=forge):
             if base_url is not None and instance.base_url != base_url:
                 continue
             try:
@@ -299,7 +307,7 @@ class cmd_my_merge_proposals(Command):
                                 ['\t%s\n' % l
                                  for l in description.splitlines()])
                         self.outf.write('\n')
-            except _mod_propose.HosterLoginRequired as e:
+            except _mod_forge.ForgeLoginRequired as e:
                 warning('Skipping %s, login required.', instance)
 
 
@@ -311,17 +319,70 @@ class cmd_land_merge_proposal(Command):
         Option('message', help='Commit message to use.', type=str)]
 
     def run(self, url, message=None):
-        proposal = _mod_propose.get_proposal_by_url(url)
+        proposal = _mod_forge.get_proposal_by_url(url)
         proposal.merge(commit_message=message)
 
 
-class cmd_hosters(Command):
+class cmd_web_open(Command):
+    __doc__ = """Open a branch page in your web browser."""
+
+    takes_options = [
+        Option('dry-run',
+               'Do not actually open the browser. Just say the URL we would '
+               'use.'),
+        ]
+    takes_args = ['location?']
+
+    def _possible_locations(self, location):
+        """Yield possible external locations for the branch at 'location'."""
+        yield location
+        try:
+            branch = _mod_branch.Branch.open_containing(location)[0]
+        except errors.NotBranchError:
+            return
+        branch_url = branch.get_public_branch()
+        if branch_url is not None:
+            yield branch_url
+        branch_url = branch.get_push_location()
+        if branch_url is not None:
+            yield branch_url
+
+    def _get_web_url(self, location):
+        for branch_url in self._possible_locations(location):
+            try:
+                branch = _mod_branch.Branch.open_containing(branch_url)[0]
+            except errors.NotBranchError as e:
+                mutter('Unable to open branch %s: %s',
+                       branch_url, e)
+                continue
+
+            try:
+                forge = _mod_forge.get_forge(branch)
+            except _mod_forge.UnsupportedForge:
+                continue
+
+            return forge.get_web_url(branch)
+        raise errors.CommandError(
+            'Unable to get web URL for %s' % location)
+
+    def run(self, location=None, dry_run=False):
+        if location is None:
+            location = u'.'
+        web_url = self._get_web_url(location)
+        note(gettext('Opening %s in web browser') % web_url)
+        if not dry_run:
+            import webbrowser
+            # otherwise brz.exe lacks this module
+            webbrowser.open(web_url)
+
+
+class cmd_forges(Command):
     __doc__ = """List all known hosting sites and user details."""
 
     hidden = True
 
     def run(self):
-        for instance in _mod_propose.iter_hoster_instances():
+        for instance in _mod_forge.iter_forge_instances():
             current_user = instance.get_current_user()
             if current_user is not None:
                 current_user_url = instance.get_user_url(current_user)
