@@ -20,9 +20,8 @@ Weave based formats scaled linearly with history size and could not represent
 ghosts.
 """
 
-from __future__ import absolute_import
-
 import gzip
+from io import BytesIO
 import os
 
 from ...lazy_import import lazy_import
@@ -44,10 +43,11 @@ from ... import (
     lockdir,
     osutils,
     trace,
-    tuned_gzip,
+    transport as _mod_transport,
     urlutils,
     )
 from ...bzr import (
+    tuned_gzip,
     versionedfile,
     weave,
     weavefile,
@@ -58,10 +58,6 @@ from ...repository import (
 from ...bzr.repository import (
     RepositoryFormatMetaDir,
     )
-from ...sixish import (
-    BytesIO,
-    text_type,
-    )
 from .store.text import TextStore
 from ...bzr.versionedfile import (
     AbsentContentFactory,
@@ -88,7 +84,7 @@ class AllInOneRepository(VersionedFileRepository):
         return xml5.serializer_v5
 
     def _escape(self, file_or_path):
-        if not isinstance(file_or_path, (str, text_type)):
+        if not isinstance(file_or_path, str):
             file_or_path = '/'.join(file_or_path)
         if file_or_path == '':
             return u''
@@ -653,14 +649,14 @@ class TextVersionedFiles(VersionedFiles):
         try:
             text = self._transport.get_bytes(path)
             compressed = self._compressed
-        except errors.NoSuchFile:
+        except _mod_transport.NoSuchFile:
             if self._compressed:
                 # try without the .gz
                 path = path[:-3]
                 try:
                     text = self._transport.get_bytes(path)
                     compressed = False
-                except errors.NoSuchFile:
+                except _mod_transport.NoSuchFile:
                     return None
             else:
                 return None
@@ -868,7 +864,7 @@ class InterWeaveRepo(InterSameDataRepository):
                     self.source._eliminate_revisions_not_present(
                         required_revisions))
             if limit is not None:
-                topo_ordered = self.get_graph().iter_topo_order(result_set)
+                topo_ordered = self.source.get_graph().iter_topo_order(result_set)
                 result_set = set(itertools.islice(topo_ordered, limit))
             return self.source.revision_ids_to_search_result(result_set)
 
