@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from typing import List, Type, Optional
 
 from breezy import (
     revision,
@@ -44,20 +45,6 @@ class InvalidRevisionSpec(errors.BzrError):
             self.extra = ''
 
 
-class InvalidRevisionSpec(errors.BzrError):
-
-    _fmt = ("Requested revision: '%(spec)s' does not exist in branch:"
-            " %(branch_url)s%(extra)s")
-
-    def __init__(self, spec, branch, extra=None):
-        errors.BzrError.__init__(self, branch=branch, spec=spec)
-        self.branch_url = getattr(branch, 'user_url', str(branch))
-        if extra:
-            self.extra = '\n' + str(extra)
-        else:
-            self.extra = ''
-
-
 class RevisionInfo(object):
     """The results of applying a revision specification to a branch."""
 
@@ -156,8 +143,8 @@ class RevisionSpec(object):
     (Equivalent to the old Branch method get_revision_info())
     """
 
-    prefix = None
-    dwim_catchable_exceptions = (InvalidRevisionSpec,)
+    prefix: Optional[str] = None
+    dwim_catchable_exceptions: List[Type[Exception]] = [InvalidRevisionSpec]
     """Exceptions that RevisionSpec_dwim._match_on will catch.
 
     If the revspec is part of ``dwim_revspecs``, it may be tried with an
@@ -300,12 +287,12 @@ class RevisionSpec_dwim(RevisionSpec):
     is called so the string describing the revision is kept here until needed.
     """
 
-    help_txt = None
+    help_txt: str
 
     _revno_regex = lazy_regex.lazy_compile(r'^(?:(\d+(\.\d+)*)|-\d+)(:.*)?$')
 
     # The revspecs to try
-    _possible_revspecs = []
+    _possible_revspecs: List[Type[registry._ObjectGetter]] = []
 
     def _try_spectype(self, rstype, branch):
         rs = rstype(self.spec, _internal=True)
@@ -320,7 +307,7 @@ class RevisionSpec_dwim(RevisionSpec):
         if self._revno_regex.match(self.spec) is not None:
             try:
                 return self._try_spectype(RevisionSpec_revno, branch)
-            except RevisionSpec_revno.dwim_catchable_exceptions:
+            except tuple(RevisionSpec_revno.dwim_catchable_exceptions):
                 pass
 
         # Next see what has been registered
@@ -328,7 +315,7 @@ class RevisionSpec_dwim(RevisionSpec):
             rs_class = objgetter.get_obj()
             try:
                 return self._try_spectype(rs_class, branch)
-            except rs_class.dwim_catchable_exceptions:
+            except tuple(rs_class.dwim_catchable_exceptions):
                 pass
 
         # Well, I dunno what it is. Note that we don't try to keep track of the
@@ -621,7 +608,7 @@ class RevisionSpec_tag(RevisionSpec):
     """
 
     prefix = 'tag:'
-    dwim_catchable_exceptions = (errors.NoSuchTag, errors.TagsNotSupported)
+    dwim_catchable_exceptions = [errors.NoSuchTag, errors.TagsNotSupported]
 
     def _match_on(self, branch, revs):
         # Can raise tags not supported, NoSuchTag, etc
@@ -797,14 +784,14 @@ class RevisionSpec_ancestor(RevisionSpec):
         from .branch import Branch
 
         with branch.lock_read():
-            revision_a = revision.ensure_null(branch.last_revision())
+            revision_a = branch.last_revision()
             if revision_a == revision.NULL_REVISION:
                 raise errors.NoCommits(branch)
             if other_location == '':
                 other_location = branch.get_parent()
             other_branch = Branch.open(other_location)
             with other_branch.lock_read():
-                revision_b = revision.ensure_null(other_branch.last_revision())
+                revision_b = other_branch.last_revision()
                 if revision_b == revision.NULL_REVISION:
                     raise errors.NoCommits(other_branch)
                 graph = branch.repository.get_graph(other_branch.repository)
@@ -826,7 +813,7 @@ class RevisionSpec_branch(RevisionSpec):
       branch:/path/to/branch
     """
     prefix = 'branch:'
-    dwim_catchable_exceptions = (errors.NotBranchError,)
+    dwim_catchable_exceptions = [errors.NotBranchError]
 
     def _match_on(self, branch, revs):
         from .branch import Branch
@@ -848,7 +835,6 @@ class RevisionSpec_branch(RevisionSpec):
         from .branch import Branch
         other_branch = Branch.open(self.spec)
         last_revision = other_branch.last_revision()
-        last_revision = revision.ensure_null(last_revision)
         context_branch.fetch(other_branch, last_revision)
         if last_revision == revision.NULL_REVISION:
             raise errors.NoCommits(other_branch)
@@ -858,7 +844,6 @@ class RevisionSpec_branch(RevisionSpec):
         from .branch import Branch
         other_branch = Branch.open(self.spec)
         last_revision = other_branch.last_revision()
-        last_revision = revision.ensure_null(last_revision)
         if last_revision == revision.NULL_REVISION:
             raise errors.NoCommits(other_branch)
         return other_branch.repository.revision_tree(last_revision)
@@ -987,7 +972,7 @@ RevisionSpec_dwim.append_possible_revspec(RevisionSpec_revid)
 RevisionSpec_dwim.append_possible_revspec(RevisionSpec_date)
 RevisionSpec_dwim.append_possible_revspec(RevisionSpec_branch)
 
-revspec_registry = registry.Registry()
+revspec_registry = registry.Registry[str, RevisionSpec]()
 
 
 def _register_revspec(revspec):
