@@ -29,6 +29,7 @@ it.
 import errno
 from io import BytesIO
 import sys
+from typing import Dict, Any, Callable
 
 from stat import S_ISDIR
 
@@ -47,7 +48,7 @@ from .. import (
 
 # a dictionary of open file streams. Keys are absolute paths, values are
 # transport defined.
-_file_streams = {}
+_file_streams: Dict[str, Any] = {}
 
 
 def _get_protocol_handlers():
@@ -467,8 +468,9 @@ class Transport(object):
     def set_segment_parameter(self, name, value):
         """Set a segment parameter.
 
-        :param name: Segment parameter name (urlencoded string)
-        :param value: Segment parameter value (urlencoded string)
+        Args:
+          name: Segment parameter name (urlencoded string)
+          value: Segment parameter value (urlencoded string)
         """
         if value is None:
             try:
@@ -503,8 +505,9 @@ class Transport(object):
         Be careful that it's not called twice, if one method is implemented on
         top of another.
 
-        :param bytes: Number of bytes read or written.
-        :param direction: 'read' or 'write' or None.
+        Args:
+          bytes: Number of bytes read or written.
+          direction: 'read' or 'write' or None.
         """
         ui.ui_factory.report_transport_activity(self, bytes, direction)
 
@@ -663,20 +666,21 @@ class Transport(object):
               upper_limit=None):
         """Get parts of the file at the given relative path.
 
-        :param relpath: The path to read data from.
-        :param offsets: A list of (offset, size) tuples.
-        :param adjust_for_latency: Adjust the requested offsets to accomodate
+        Args:
+          relpath: The path to read data from.
+          offsets: A list of (offset, size) tuples.
+          adjust_for_latency: Adjust the requested offsets to accomodate
             transport latency. This may re-order the offsets, expand them to
             grab adjacent data when there is likely a high cost to requesting
             data relative to delivering it.
-        :param upper_limit: When adjust_for_latency is True setting upper_limit
+          upper_limit: When adjust_for_latency is True setting upper_limit
             allows the caller to tell the transport about the length of the
             file, so that requests are not issued for ranges beyond the end of
             the file. This matters because some servers and/or transports error
             in such a case rather than just satisfying the available ranges.
             upper_limit should always be provided when adjust_for_latency is
             True, and should be the size of the file in bytes.
-        :return: A list or generator of (offset, data) tuples
+        Returns: A list or generator of (offset, data) tuples
         """
         if adjust_for_latency:
             # Design note: We may wish to have different algorithms for the
@@ -1363,12 +1367,13 @@ class ConnectedTransport(Transport):
         user, password, host and path will be quoted if they contain reserved
         chars.
 
-        :param scheme: protocol
-        :param user: login
-        :param password: associated password
-        :param host: the server address
-        :param port: the associated port
-        :param path: the absolute path on the server
+        Args:
+          scheme: protocol
+          user: login
+          password: associated password
+          host: the server address
+          port: the associated port
+          path: the absolute path on the server
 
         :return: The corresponding URL.
         """
@@ -1407,7 +1412,8 @@ class ConnectedTransport(Transport):
     def abspath(self, relpath):
         """Return the full url to the given relative path.
 
-        :param relpath: the relative path urlencoded
+        Args:
+          relpath: the relative path urlencoded
 
         :returns: the Unicode version of the absolute path for relpath.
         """
@@ -1420,7 +1426,8 @@ class ConnectedTransport(Transport):
         requests, daughter classes should redefine this method if needed and
         use the result to build their requests.
 
-        :param relpath: the path relative to the transport base urlencoded.
+        Args:
+          relpath: the path relative to the transport base urlencoded.
 
         :return: the absolute Unicode path on the server,
         """
@@ -1444,10 +1451,10 @@ class ConnectedTransport(Transport):
         always call this method to set the connection and do so each time a new
         connection is created.
 
-        :param connection: An opaque object representing the connection used by
+        Args:
+          connection: An opaque object representing the connection used by
             the daughter class.
-
-        :param credentials: An opaque object representing the credentials
+          credentials: An opaque object representing the credentials
             needed to create the connection.
         """
         self._shared_connection.connection = connection
@@ -1539,11 +1546,12 @@ def get_transport_from_path(path, possible_transports=None):
 def get_transport_from_url(url, possible_transports=None):
     """Open a transport to access a URL.
 
-    :param base: a URL
-    :param transports: optional reusable transports list. If not None, created
+    Args:
+      base: a URL
+      transports: optional reusable transports list. If not None, created
         transports will be added to the list.
 
-    :return: A new transport optionally sharing its connection with one of
+    Returns: A new transport optionally sharing its connection with one of
         possible_transports.
     """
     transport = None
@@ -1574,12 +1582,11 @@ def get_transport_from_url(url, possible_transports=None):
 def get_transport(base, possible_transports=None, purpose=None):
     """Open a transport to access a URL or directory.
 
-    :param base: either a URL or a directory name.
-
-    :param transports: optional reusable transports list. If not None, created
+    Args:
+      base: either a URL or a directory name.
+      transports: optional reusable transports list. If not None, created
         transports will be added to the list.
-
-    :param purpose: Purpose for which the transport will be used
+      purpose: Purpose for which the transport will be used
         (e.g. 'read', 'write' or None)
 
     :return: A new transport optionally sharing its connection with one of
@@ -1614,10 +1621,11 @@ def do_catching_redirections(action, transport, redirected):
     inform the user about each redirection or only inform the user of a user
     via the exception parameter.
 
-    :param action: A callable, what the caller want to do while catching
+    Args:
+      action: A callable, what the caller want to do while catching
                   redirections.
-    :param transport: The initial transport used.
-    :param redirected: A callable receiving the redirected transport and the
+      transport: The initial transport used.
+      redirected: A callable receiving the redirected transport and the
                   RedirectRequested exception.
 
     :return: Whatever 'action' returns
@@ -1769,7 +1777,7 @@ register_lazy_transport('ssh:', 'breezy.transport.remote',
                         'HintingSSHTransport')
 
 
-transport_server_registry = registry.Registry()
+transport_server_registry = registry.Registry[str, Callable]()
 transport_server_registry.register_lazy('bzr', 'breezy.bzr.smart.server',
                                         'serve_bzr', help="The Bazaar smart server protocol over TCP. (default port: 4155)")
 transport_server_registry.default_key = 'bzr'
